<?php
/**
 * --------------------------------------------------------------------------------
 * Payment Plugin - Payanyway
 * --------------------------------------------------------------------------------
 * @package     Joomla 2.5 -  3.x
 * @subpackage  J2Store
 * @author      J2Store <support@j2store.org>
 * @copyright   Copyright (c) 2014-19 J2Store . All rights reserved.
 * @license     GNU/GPL license: http://www.gnu.org/licenses/gpl-2.0.html
 * --------------------------------------------------------------------------------
 *
 * */

// No direct access

defined('_JEXEC') or die('Restricted access');

require_once JPATH_ADMINISTRATOR . '/components/com_j2store/library/plugins/payment.php';
require_once JPATH_ADMINISTRATOR . '/components/com_j2store/helpers/j2store.php';


class plgJ2StorePayment_payanyway extends J2StorePaymentPlugin
{
	/**
	 * @var string
	 */
	public $_element = 'payment_payanyway';
	public $code_arr = array();
	private $_isLog = true;
	var $_j2version = null;

	/**
	 * @param object $subject
	 * @param array $config
	 */
	function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);
	}

	/**
	 * @param array $data
	 * @return string
	 */
	public function _prePayment( $data )
	{
		$app = JFactory::getApplication();
		$currency = J2Store::currency();

		// Prepare the payment form
		$vars = new JObject;

		$vars->url = JRoute::_("index.php?option=com_j2store&view=checkout");
		$vars->order_id = $data['order_id'];
		$vars->orderpayment_id = $data['orderpayment_id'];
		$vars->orderpayment_type = $this->_element;

		F0FTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_j2store/tables');
		$order = F0FTable::getInstance('Order', 'J2StoreTable');
		$order->load($data['orderpayment_id']);

		$currency_values= $this->getCurrency($order);
		$amount = J2Store::currency()->format($order->order_total, $currency_values['currency_code'], $currency_values['currency_value'], false);

		$vars->amount = $amount*100;
		$vars->currency_code = $currency_values['currency_code'];

		$vars->display_name = $this->params->get('display_name', 'PLG_J2STORE_PAYMENT_PAYANYWAY');
		$vars->onbeforepayment_text = $this->params->get('onbeforepayment', '');
		$vars->button_text = $this->params->get('button_text', 'J2STORE_PLACE_ORDER');
		$vars->sandbox = $this->params->get('sandbox', 0);

		// Lets check the values submitted
		$html = $this->_getLayout('prepayment', $vars);

		return $html;
	}

	const paw_kassa_VAT0     = '1104';  // НДС 0%
	const paw_kassa_VAT10    = '1103';  // НДС 10%
	const paw_kassa_VAT18    = '1102';  // НДС 18%
	const paw_kassa_VATNOVAT = '1105';  // НДС не облагается
	const paw_kassa_VATWR10  = '1107';  // НДС с рассч. ставкой 10%
	const paw_kassa_VATWR18  = '1106';  // НДС с рассч. ставкой 18%

	private static function monetaPayURLResponse($mnt_id, $mnt_transaction_id, $mnt_data_integrity_code, $success = false,
												 $repeatRequest = false, $echo = true, $die = true, $kassa_inventory = null, $kassa_customer = null, $kassa_delivery = null)
	{
		if ($success === true)
			$resultCode = '200';
		elseif ($repeatRequest === true)
			$resultCode = '402';
		else
			$resultCode = '500';
		$mnt_signature = md5($resultCode.$mnt_id.$mnt_transaction_id.$mnt_data_integrity_code);
		$response = '<?xml version="1.0" encoding="UTF-8" ?>'."\n";
		$response .= "<MNT_RESPONSE>\n";
		$response .= "<MNT_ID>{$mnt_id}</MNT_ID>\n";
		$response .= "<MNT_TRANSACTION_ID>{$mnt_transaction_id}</MNT_TRANSACTION_ID>\n";
		$response .= "<MNT_RESULT_CODE>{$resultCode}</MNT_RESULT_CODE>\n";
		$response .= "<MNT_SIGNATURE>{$mnt_signature}</MNT_SIGNATURE>\n";
		if (!empty($kassa_inventory) || !empty($kassa_customer) || !empty($kassa_delivery))
		{
			$response .= "<MNT_ATTRIBUTES>\n";
			foreach (array('INVENTORY' => $kassa_inventory, 'CUSTOMER' => $kassa_customer, 'DELIVERY' => $kassa_delivery) as $k => $v)
				if (!empty($v))
					$response .= "<ATTRIBUTE><KEY>{$k}</KEY><VALUE>{$v}</VALUE></ATTRIBUTE>\n";
			$response .= "</MNT_ATTRIBUTES>\n";
		}
		$response .= "</MNT_RESPONSE>\n";
		if ($echo === true)
		{
			header("Content-type: application/xml");
			echo $response;
		}
		else
			return $response;
		if ($die === true)
			die;
		return '';
	}


	public function _postPayment( $data )
	{
		$app = JFactory::getApplication();
		$paction = $app->input->getString('paction');

		if ($paction == 'process') {

			$paw_account_id = trim($this->params->get('paw_account_id'));
			$paw_account_code = trim($this->params->get('paw_account_code'));
			$paw_kassa = $this->params->get('paw_kassa');

			$MNT_ID 		    = $this->getVarFromRequest('MNT_ID');
			$MNT_TRANSACTION_ID = $this->getVarFromRequest('MNT_TRANSACTION_ID');
			$MNT_OPERATION_ID 	= $this->getVarFromRequest('MNT_OPERATION_ID');
			$MNT_AMOUNT 		= $this->getVarFromRequest('MNT_AMOUNT');
			$MNT_CURRENCY_CODE 	= $this->getVarFromRequest('MNT_CURRENCY_CODE');
			$MNT_TEST_MODE 		= $this->getVarFromRequest('MNT_TEST_MODE');
			$MNT_SIGNATURE 		= $this->getVarFromRequest('MNT_SIGNATURE');

			if ($paw_account_code) {
                $paw_signature = md5( $MNT_ID . $MNT_TRANSACTION_ID . $MNT_OPERATION_ID . $MNT_AMOUNT . $MNT_CURRENCY_CODE . $MNT_TEST_MODE . $paw_account_code );
                if ($MNT_SIGNATURE != $paw_signature || $paw_account_id != $MNT_ID) {
                    echo "FAIL";
                    die();
                }
			}

            if (strpos($MNT_TRANSACTION_ID, '_') !== false) {
                $transactionIdArray = explode('_', $MNT_TRANSACTION_ID);
                $MNT_TRANSACTION_ID = $transactionIdArray[0];
            }

            // get order information
            F0FTable::addIncludePath( JPATH_ADMINISTRATOR.'/components/com_j2store/tables' );
			/** @var J2StoreTableProduct $product */
			$product = F0FTable::getInstance('Tax', 'J2StoreTable');
			/** @var J2StoreTableOrder $order */
            $order = F0FTable::getInstance('Order', 'J2StoreTable');
            if ($order->load( array('order_id' => $MNT_TRANSACTION_ID))) {
                if ( !empty($order->order_id) ) {
                    $order->transaction_id       = $MNT_OPERATION_ID;
                    $order->transaction_status   = 'OK';
                    $order->payment_complete();
                    $order->store();

					if ($paw_kassa == '1')
					{
						$orderTotals = $order->get_formatted_order_totals();
						$kassa_delivery = floatval(@$orderTotals['shipping']['value']) + floatval(@$orderTotals['shipping_tax']['value']);
						$user_email = $order->user_email;
						/** @var J2StoreTableOrderItem[] $items */
						$items = $order->getItems();
						$taxRates = $order->getOrderTaxRates();
						$inventoryPositions = array();
						/** @var J2StoreTableOrderItem $item */
						foreach ($items as $n => $item)
						{
							$itemData = $item->getData();
							$taxRate = 0;
							foreach ($taxRates as $t)
							{
								if ($t->j2store_ordertax_id == $itemData['j2store_orderitem_id'] &&
									$t->order_id == $itemData['order_id']
								)
								{
									$taxRate = intval($t->ordertax_percent);
									break;
								}
							}
							$vatTag = self::paw_kassa_VATNOVAT;
							if ($taxRate == 18)
								$vatTag = self::paw_kassa_VATWR18;
							if ($taxRate == 10)
								$vatTag = self::paw_kassa_VATWR10;
							$inventoryPositions[] = array(
								'name' => trim(preg_replace("/&?[a-z0-9]+;/i", "", htmlspecialchars($item->orderitem_name))),
								'price' => floatval($item->orderitem_price) + floatval($item->orderitem_per_item_tax),
								'quantity' => $item->orderitem_quantity,
								'vatTag' => $vatTag,
								'taxRate' => $taxRate
							);
						}
						$kassa_inventory = json_encode($inventoryPositions);
						self::monetaPayURLResponse($MNT_ID, $MNT_TRANSACTION_ID, $paw_account_code, true, false, true,
							true, $kassa_inventory, $user_email, $kassa_delivery);								// 200
					}

					echo "SUCCESS";
					die();
                }
            }

            echo "FAIL";
			$app->close();
			die();

		}
        else if (isset($data['order_id']) && isset($data['order_state_id']) && $data['order_state_id'] == 1 && isset($_POST['amount']) && isset($_POST['currency_code'])) {
            $order_id = $data['order_id'];
            $order_amount = number_format(intval($_POST['amount']) / 100, 2, '.', '');
            $order_currency_code = $_POST['currency_code'];
            $paw_account_id = trim($this->params->get('paw_account_id'));
            $paw_account_code = trim($this->params->get('paw_account_code'));
            $sandbox = trim($this->params->get('sandbox'));
			$paw_server = $this->params->get('paw_server');

            if (!$sandbox) {
                $sandbox = "0";
            }

            // test payment at demo.moneta
            // $redirect_url = 'http://demo.moneta.ru/assistant.htm';
            // pay in production
            $redirect_url = 'https://'.$paw_server.'/assistant.htm';
			//$redirect_url = 'http://localhost:8080/assistant.htm';

            // add rnd to proper order_id
            $transaction_id = $order_id . '_' . time();
            $paw_signature = md5( $paw_account_id . $transaction_id . $order_amount . $order_currency_code . $sandbox . $paw_account_code );

            $redirect_url .= '?MNT_ID=' . $paw_account_id;
            $redirect_url .= '&MNT_AMOUNT=' . $order_amount;
            $redirect_url .= '&MNT_TRANSACTION_ID=' . $transaction_id;
            $redirect_url .= '&MNT_CURRENCY_CODE=' . $order_currency_code;
            $redirect_url .= '&MNT_SIGNATURE=' . $paw_signature;

            header("Location: {$redirect_url}");
			$app->close();
            die();
        }
	}

	/**
	 * @param array $data
	 * @return string
	 */
	public function _renderForm($data)
	{
		$vars = new JObject();
		$vars->prepop = array();
		$vars->onselection_text = $this->params->get('onselection', '');
		$html = $this->_getLayout('form', $vars);

		return $html;
	}

	/**
	 * @param array $submitted_values
	 * @return JObject
	 */
	public function _verifyForm($submitted_values)
	{
		$object = new JObject();
		$object->error = false;
		$object->message = '';

    	return $object;
    }

	/**
	 * @return array
	 */
	public function _process() {

		$app = JFactory::getApplication ();
		$data = $app->input->getArray ( $_POST );
		$json = array ();
		$errors = array ();

		return $json;
	}

	/**
	 * @param $varName
	 * @return null
	 */
	private function getVarFromRequest($varName) {
		$value = null;
		if (isset($_GET[$varName])) {
			$value = $_GET[$varName];
		}
		if (isset($_POST[$varName])) {
			$value = $_POST[$varName];
		}
		return $value;
	}

	/**
	 * @param $text
	 * @param string $type
	 */
	public function _log($text, $type = 'message')
	{
		if ($this->_isLog)
		:
		{
			$file = JPATH_ROOT . "/cache/{$this->_element}.log";
			$date = JFactory::getDate();

			$f = fopen($file, 'a');
			fwrite($f, "\n\n" . $date->format('Y-m-d H:i:s'));
			fwrite($f, "\n" . $type . ': ' . $text);
			fclose($f);
		}
		endif;
	}


}
